package com.tzld.piaoquan.longarticle.service.local.impl;

import com.alibaba.fastjson.JSONObject;
import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.tzld.piaoquan.longarticle.model.bo.VideoDetail;
import com.tzld.piaoquan.longarticle.model.po.PublishMiniprogram;
import com.tzld.piaoquan.longarticle.service.local.CardService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.*;

@Slf4j
@Service
public class CardServiceImpl implements CardService {

    @ApolloJsonValue("${miniMap:{}}")
    private JSONObject miniProgramMap;


    public JSONObject createGzhPath(String videoId, String sharedUid, String ghId, Integer miniProgramType) throws UnsupportedEncodingException {
        String rootShareId = UUID.randomUUID().toString();
        String rootSourceId;
        switch (miniProgramType) {
            case 1:
                rootSourceId = "longArticles_" + generateSourceId();
                break;
            case 2:
                rootSourceId = "touliu_tencentGzhArticle_" + ghId + "_" + generateSourceId();
                break;
            case 3:
                rootSourceId = "WeCom_" + generateSourceId();
                break;
            default:
                rootSourceId = "Error mini_program_type " + miniProgramType;
        }

        String url = String.format("pages/user-videos?id=%s&su=%s&fromGzh=1&rootShareId=%s&shareId=%s&rootSourceId=%s",
                videoId, sharedUid, rootShareId, rootShareId, rootSourceId);
        String productionPath = String.format("pages/category?jumpPage=%s", URLEncoder.encode(url, StandardCharsets.UTF_8.toString()));
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("rootShareId", rootShareId);
        jsonObject.put("rootSourceId", rootSourceId);
        jsonObject.put("productionPath", productionPath);
        return jsonObject;
    }

    private String generateSourceId() {
        long timestamp = System.currentTimeMillis();
        int randomNum = new Random().nextInt(9000) + 1000;
        String hashInput = timestamp + "-" + randomNum;
        return md5(hashInput);
    }

    private String md5(String input) {
        try {
            MessageDigest md = MessageDigest.getInstance("MD5");
            byte[] messageDigest = md.digest(input.getBytes(StandardCharsets.UTF_8));
            StringBuilder sb = new StringBuilder();
            for (byte b : messageDigest) {
                sb.append(String.format("%02x", b));
            }
            return sb.toString();
        } catch (NoSuchAlgorithmException e) {
            throw new RuntimeException(e);
        }
    }

    public PublishMiniprogram generateSingleCard(Long planAccountId, Integer index, String ghId, Integer miniId, VideoDetail videoDetail, Integer miniProgramType, String publishContentId) throws UnsupportedEncodingException {
        String strMiniId = String.valueOf(miniId);
        JSONObject miniInfo = miniProgramMap.getJSONObject(strMiniId);
        String avatar = miniInfo.getString("avatar");
        String appId = miniInfo.getString("id");
        String appName = miniInfo.getString("name");

        String videoId = videoDetail.getVideoId();
        String uid = videoDetail.getUid();
        JSONObject jsonObject = createGzhPath(videoId, uid, ghId, miniProgramType);


        PublishMiniprogram publishMiniprogram = new PublishMiniprogram();
        publishMiniprogram.setPlanAccountId(planAccountId);
        publishMiniprogram.setAppId(appId);
        publishMiniprogram.setPublishContentId(publishContentId);
        publishMiniprogram.setAppId(appId);
        publishMiniprogram.setAppName(appName);
        publishMiniprogram.setAvatar(avatar);
        publishMiniprogram.setMiniId(miniId);
        publishMiniprogram.setMiniProgramType(miniProgramType);
        publishMiniprogram.setVideoId(Long.parseLong(videoId));
        publishMiniprogram.setSource(videoDetail.getSource());
        publishMiniprogram.setVideoTitle(videoDetail.getKimiTitle());
        publishMiniprogram.setVideoCover(videoDetail.getVideoCover());
        publishMiniprogram.setVideoPath(videoDetail.getVideoPath());
        publishMiniprogram.setProductionPath(jsonObject.getString("productionPath"));
        publishMiniprogram.setRootSourceId(jsonObject.getString("rootSourceId"));
        publishMiniprogram.setRootShareId(jsonObject.getString("rootShareId"));
        publishMiniprogram.setIndex(index);
        publishMiniprogram.setTraceId(videoDetail.getTraceId());
        return publishMiniprogram;
    }

    public List<PublishMiniprogram> generateCards(Long planAccountId, String ghId, List<VideoDetail> videoDetails, int miniProgramType, String publishContentId) {
        try {
            int longArticlesMiniProgramId = 25;
            int touliuMiniProgramId = 33;
            int weComMiniProgramId = 27;
            int miniId = 0;

            List<PublishMiniprogram> cardList = new ArrayList<>();

            switch (miniProgramType) {
                case 1:
                    miniId = longArticlesMiniProgramId;
                    break;
                case 2:
                    miniId = touliuMiniProgramId;
                    break;
                case 3:
                    miniId = weComMiniProgramId;
                    break;
                default:
                    break;
            }
            if (miniId == 0) {
                return null;
            }
            for (int index = 0; index < videoDetails.size(); index++) {
                VideoDetail videoDetail = videoDetails.get(index);
                PublishMiniprogram card = generateSingleCard(planAccountId, index + 1, ghId, miniId, videoDetail, miniProgramType, publishContentId);
                if (card != null) {
                    cardList.add(card);
                }
            }
            return cardList;
        } catch (UnsupportedEncodingException e) {
            log.error("generateCards error {}", e.getMessage());
        }
        return null;
    }
}
