package com.tzld.piaoquan.longarticle.service.local.impl;

import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.tzld.piaoquan.longarticle.dao.mapper.CrawlerVideoMapper;
import com.tzld.piaoquan.longarticle.dao.mapper.MatchVideoMapper;
import com.tzld.piaoquan.longarticle.dao.mapper.PublishContentMapper;
import com.tzld.piaoquan.longarticle.dao.mapper.PublishMiniprogramMapper;
import com.tzld.piaoquan.longarticle.model.dto.PublishArticleData;
import com.tzld.piaoquan.longarticle.model.po.*;
import com.tzld.piaoquan.longarticle.service.local.ContentService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

@Slf4j
@Service
public class ContentServiceImpl implements ContentService {
    @ApolloJsonValue("${testAccountLevel2:{}}")
    private Map<String, Integer> ghIdMap;

    @Autowired
    private MatchVideoMapper matchVideoMapper;

    @Autowired
    private CrawlerVideoMapper crawlerVideoMapper;

    @Autowired
    private PublishMiniprogramMapper publishMiniprogramMapper;

    @Autowired
    private RedisTemplate<String, Object> redisTemplate;

    public MatchVideo getContent(String contentId, String ghId) {
        MatchVideoExample matchVideoExample = new MatchVideoExample();
        matchVideoExample.createCriteria().andGhIdEqualTo(ghId).andContentIdEqualTo(contentId);
        List<MatchVideo> matchVideos = matchVideoMapper.selectByExample(matchVideoExample);
        if (CollectionUtils.isEmpty(matchVideos)) {
            return null;
        }
        return matchVideos.get(0);
    }

    public List<PublishMiniprogram> getPublishMiniprograms(PublishArticleData publishArticleData) {
        PublishMiniprogramExample publishMiniprogramExample = new PublishMiniprogramExample();
        publishMiniprogramExample.createCriteria().andPublishContentIdEqualTo(publishArticleData.getId());
        List<PublishMiniprogram> publishMiniprograms = publishMiniprogramMapper.selectByExample(publishMiniprogramExample);
        if (CollectionUtils.isEmpty(publishMiniprograms)) {
            return null;
        }
        return publishMiniprograms;
    }


    public List<CrawlerVideo> getContentMiniVideo(PublishArticleData publishArticleData, PlanAccount planAccount) {
        CrawlerVideoExample example = new CrawlerVideoExample();
        example.setOrderByClause("score desc");
        example.createCriteria().andDownloadStatusEqualTo(2).andContentIdEqualTo(publishArticleData.getSourceId());
        List<CrawlerVideo> crawlerVideoList = crawlerVideoMapper.selectByExample(example);
        List<CrawlerVideo> videoList;
        switch (publishArticleData.getContentPoolType()) {
            case "autoArticlePoolLevel4":
                // 冷启层，全量做
                videoList = shuffleList(crawlerVideoList).subList(0, Math.min(crawlerVideoList.size(), 3));
                break;
            case "autoArticlePoolLevel3":
                //次条，只针对具体账号做
                if (ghIdMap.containsKey(planAccount.getGhId())) {
                    videoList = shuffleList(crawlerVideoList).subList(0, Math.min(crawlerVideoList.size(), 3));
                } else {
                    videoList = crawlerVideoList.subList(0, Math.min(crawlerVideoList.size(), 3));
                }
                break;
            case "autoArticlePoolLevel1":
                // 头条，先不做
                videoList = crawlerVideoList.subList(0, Math.min(crawlerVideoList.size(), 3));
                break;
            default:
                videoList = crawlerVideoList.subList(0, Math.min(crawlerVideoList.size(), 3));
                break;
        }

        return videoList;
    }

    private <T> List<T> shuffleList(List<T> oriList) {
        // 使用 ArrayList 创建副本
        List<T> shuffledList = new ArrayList<>(oriList);
        Collections.shuffle(shuffledList); // 随机打乱列表
        return shuffledList;
    }

}
