package com.tzld.piaoquan.longarticle.service.local.impl;

import com.alibaba.fastjson.JSON;
import com.tzld.piaoquan.longarticle.common.enums.ContentStatusEnum;
import com.tzld.piaoquan.longarticle.common.enums.PublishGzhPushTypeEnum;
import com.tzld.piaoquan.longarticle.dao.mapper.*;
import com.tzld.piaoquan.longarticle.model.bo.MatchContent;
import com.tzld.piaoquan.longarticle.model.bo.VideoDetail;
import com.tzld.piaoquan.longarticle.model.dto.*;
import com.tzld.piaoquan.longarticle.model.po.*;
import com.tzld.piaoquan.longarticle.model.vo.*;
import com.tzld.piaoquan.longarticle.service.local.CoreService;
import com.tzld.piaoquan.longarticle.service.local.PlanAccountService;
import com.tzld.piaoquan.longarticle.service.remote.AigcService;
import com.tzld.piaoquan.longarticle.service.remote.MatchService;
import com.tzld.piaoquan.longarticle.service.remote.VideoService;
import com.tzld.piaoquan.longarticle.service.remote.impl.SortServiceImpl;
import com.tzld.piaoquan.longarticle.utils.DateUtil;
import com.tzld.piaoquan.longarticle.utils.LarkRobotUtil;
import com.tzld.piaoquan.longarticle.utils.TimeZoneUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.time.LocalTime;
import java.util.*;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

@Slf4j
@Service
public class CoreServiceImpl implements CoreService {

    @Autowired
    private AigcService aigcService;

    @Autowired
    private SortServiceImpl sortService;

    @Autowired
    private ContentServiceImpl contentService;

    @Autowired
    private MatchService matchService;

    @Autowired
    private VideoService videoService;

    @Autowired
    private KimiServiceImpl kimiService;

    @Autowired
    private CardServiceImpl cardService;

    @Autowired
    private PlanAccountMapper planAccountMapper;

    @Autowired
    private PlanAccountService planAccountService;

    @Autowired
    private PublishMiniprogramMapper publishMiniprogramMapper;

    @Autowired
    private PublishContentMapper publishContentMapper;

    @Autowired
    private RootSourceMapper rootSourceMapper;

    @Autowired
    private PublicContentServiceImpl publicContentService;

    @Autowired
    private RedisTemplate<String, Object> redisTemplate;

    @Override
    public void initPlanAccount() {
        List<Plan> planList = planAccountService.getPlanList();
        Set<String> planIdSet = planList.stream().map(Plan::getPlanId).collect(Collectors.toSet());
        List<LongArticleSystemPlan> allLongArticleSystemPlans = aigcService.getAllLongArticleSystemPlan();
        for (LongArticleSystemPlan longArticleSystemPlan : allLongArticleSystemPlans) {
            Plan plan = createPlan(longArticleSystemPlan);
            planAccountService.saveOrUpdatePlan(plan);
            planIdSet.remove(longArticleSystemPlan.getId());
            //获取排序策略
            Map<String, String> sortStgMap = new HashMap<>();
            List<GzhArticleSortTaskParam> gzhArticleSortTask = longArticleSystemPlan.getGzhArticleSortTask();
            for (GzhArticleSortTaskParam gzhArticleSortTaskParam : gzhArticleSortTask) {
                String strategy = gzhArticleSortTaskParam.getStrategy();
                List<String> accountIds = gzhArticleSortTaskParam.getAccountIds();
                for (String accountId : accountIds) {
                    sortStgMap.put(accountId, strategy);
                }
            }
            //获取小程序策略
            Map<String, MiniprogramInsertTaskParam> miniStgMap = new HashMap<>();
            List<MiniprogramInsertTaskParam> miniprogramInsertTasks = longArticleSystemPlan.getMiniprogramInsertTasks();
            for (MiniprogramInsertTaskParam miniprogramInsertTaskParam : miniprogramInsertTasks) {
                List<String> accountIds = miniprogramInsertTaskParam.getAccountIds();
                for (String accountId : accountIds) {
                    miniStgMap.put(accountId, miniprogramInsertTaskParam);
                }
            }
            for (LongArticleSystemPlanAccount account : longArticleSystemPlan.getAccountList()) {
                PlanAccount planAccount = new PlanAccount();
                String accountId = account.getId();
                planAccount.setPlanId(longArticleSystemPlan.getId());
                planAccount.setPlanName(longArticleSystemPlan.getName());
                planAccount.setAccountId(accountId);
                planAccount.setAccountName(account.getName());
                planAccount.setGhId(account.getGhId());
                planAccount.setPublishRate(longArticleSystemPlan.getPublishRate());
                planAccount.setPublishDate(longArticleSystemPlan.getPublishDate());
                planAccount.setPublishNum(longArticleSystemPlan.getPublishNum());
                planAccount.setPublishWindowStart(longArticleSystemPlan.getPublishWindowStart());
                planAccount.setPublishWindowEnd(longArticleSystemPlan.getPublishWindowEnd());
                planAccount.setPublishTimeInterval(longArticleSystemPlan.getPublishTimeInterval());
                planAccount.setPublishNum(longArticleSystemPlan.getPublishNum());
                planAccount.setPublishPreNum(longArticleSystemPlan.getPublishPerNum());
                planAccount.setPublishPreMinNum(longArticleSystemPlan.getPublishPerMinNum());
                planAccount.setPublishPushTime(longArticleSystemPlan.getPublishPushTime());
                planAccount.setPushType(longArticleSystemPlan.getPushType());
                planAccount.setMiniprogramInsertFlag(longArticleSystemPlan.getMiniprogramInsertFlag());
                MiniprogramInsertTaskParam miniprogramInsertTaskParam = miniStgMap.get(accountId);
                if (miniprogramInsertTaskParam != null) {
                    planAccount.setMiniprogramUseType(miniprogramInsertTaskParam.getMiniprogramUseType());
                    planAccount.setMiniStrategy(miniprogramInsertTaskParam.getStrategy());
                }
                if (sortStgMap.containsKey(accountId)) {
                    planAccount.setSortStrategy(sortStgMap.get(accountId));
                }
                planAccount.setPushType(longArticleSystemPlan.getPushType());
                planAccountService.saveOrUpdatePlanAccount(planAccount);
            }
        }
        if (CollectionUtils.isEmpty(planIdSet)) {
            return;
        }
        for (String planId : planIdSet) {
            planAccountService.delPlan(planId);
        }
    }

    private Plan createPlan(LongArticleSystemPlan longArticleSystemPlan) {
        Plan plan = new Plan();
        plan.setPlanId(longArticleSystemPlan.getId());
        plan.setPlanName(longArticleSystemPlan.getName());
        plan.setPublishRate(longArticleSystemPlan.getPublishRate());
        plan.setPublishDate(longArticleSystemPlan.getPublishDate());
        plan.setPublishWindowStart(longArticleSystemPlan.getPublishWindowStart());
        plan.setPublishWindowEnd(longArticleSystemPlan.getPublishWindowEnd());
        plan.setPublishTimeInterval(longArticleSystemPlan.getPublishTimeInterval());
        plan.setPublishNum(longArticleSystemPlan.getPublishNum());
        plan.setPublishPreNum(longArticleSystemPlan.getPublishPerNum());
        plan.setPublishPreMinNum(longArticleSystemPlan.getPublishPerMinNum());
        plan.setPublishPushTime(longArticleSystemPlan.getPublishPushTime());
        plan.setPushType(longArticleSystemPlan.getPushType());
        plan.setMiniprogramInsertFlag(longArticleSystemPlan.getMiniprogramInsertFlag());
        return plan;
    }


    // 定义一个阻塞队列
    private static final ArrayBlockingQueue<MatchContent> queue = new ArrayBlockingQueue<>(100000);

    private static final int size = 5;
    // 定义一个线程池，设置消费线程的数量
    private static ExecutorService executorService = Executors.newFixedThreadPool(size);


    @Override
    public void matchContent() {

        if (executorService.isShutdown() || executorService.isTerminated()) {
            executorService = Executors.newFixedThreadPool(size);
        } else {
            executorService.shutdownNow();
            executorService = Executors.newFixedThreadPool(size);
        }

        // 启动生产者线程
        new Thread(() -> {
            List<PlanAccount> matchPlanAccounts = planAccountService.getMatchPlanAccount();
            if (CollectionUtils.isEmpty(matchPlanAccounts)) {
                return;
            }
            for (PlanAccount planAccount : matchPlanAccounts) {
                LongArticleSystemGetContentsParam param = new LongArticleSystemGetContentsParam();
                param.setAccountId(planAccount.getAccountId());
                param.setPlanId(planAccount.getPlanId());
                LongArticleSystemContentVO longArticleSystemContentVO = aigcService.getContentItemList(param);
                if (longArticleSystemContentVO == null || CollectionUtils.isEmpty(longArticleSystemContentVO.getContentItemList())) {
                    continue;
                }
                for (ContentItemVO contentItemVO : longArticleSystemContentVO.getContentItemList()) {
                    MatchContent matchContent = new MatchContent();
                    matchContent.setSourceId(contentItemVO.getSourceId());
                    matchContent.setGhId(planAccount.getGhId());
                    matchContent.setPublishContentId(contentItemVO.getPublishContentId());
                    matchContent.setAccountName(planAccount.getAccountName());
                    matchContent.setContent(contentItemVO.getContent());
                    matchContent.setTitle(contentItemVO.getTitle());
                    matchContent.setFlowPoolLevelTag(contentItemVO.getFlowPoolLevelTag());
                    try {
                        queue.put(matchContent);
                    } catch (InterruptedException e) {
                        throw new RuntimeException(e);
                    }
                }
            }
        }).start();


        // 启动消费者线程
        for (int i = 0; i < size; i++) {
            executorService.submit(new Thread(() -> {
                while (true) {
                    try {
                        MatchContent matchContent = queue.take();
                        MatchVideo content = contentService.getContent(matchContent.getSourceId(), matchContent.getGhId());
                        if (content != null) {
                            Integer contentStatus = content.getContentStatus();
                            if (ContentStatusEnum.isSuccess(contentStatus)) {
                                MatchMiniprogramStatusParam statusParam = new MatchMiniprogramStatusParam();
                                statusParam.setStatus(2);
                                statusParam.setPublishContentId(matchContent.getPublishContentId());
                                aigcService.updateMatchMiniprogramStatus(statusParam);
                                continue;
                            }
                            if (ContentStatusEnum.isFail(contentStatus)) {
                                MatchMiniprogramStatusParam statusParam = new MatchMiniprogramStatusParam();
                                statusParam.setStatus(3);
                                statusParam.setPublishContentId(matchContent.getPublishContentId());
                                String errorMessage = ContentStatusEnum.getErrorMessage(contentStatus);
                                statusParam.setErrorMsg(errorMessage);
                                aigcService.updateMatchMiniprogramStatus(statusParam);
                            }
                        } else {
                            MiniprogramCardRequest request = new MiniprogramCardRequest();
                            request.setGhId(matchContent.getGhId());
                            request.setAccountName(matchContent.getAccountName());
                            request.setContent(matchContent.getContent());
                            request.setTitle(matchContent.getTitle());
                            //请求到新服务
                            request.setStrategy("strategy_v2");
                            request.setArticleId(matchContent.getSourceId());
                            request.setFlowPoolLevelTag(matchContent.getFlowPoolLevelTag());
                            matchService.matchMiniprogramVideo(request);
                        }
                    } catch (InterruptedException e) {
                        throw new RuntimeException(e);
                    }
                }
            }));
        }
    }


    private boolean effectiveTime(String startWindow, String endWindow, TimeZoneUtil.Timezone timezone) {
        if (!org.springframework.util.StringUtils.hasText(startWindow) || !org.springframework.util.StringUtils.hasText(endWindow)) {
            return true;
        }
        LocalTime currentTime = TimeZoneUtil
                .currentTime(Optional.ofNullable(timezone).orElse(TimeZoneUtil.Timezone.china));
        return currentTime.isAfter(LocalTime.parse(startWindow)) && currentTime.isBefore(LocalTime.parse(endWindow));
    }

    private boolean checkPlanAccount(PlanAccount planAccount) {
        boolean isGzhGroupPushPlan = Objects.nonNull(planAccount.getPushType())
                && (PublishGzhPushTypeEnum.auto_group_publish.getVal().equals(planAccount.getPushType())
                || PublishGzhPushTypeEnum.batch_group_publish.getVal().equals(planAccount.getPushType()));
        boolean flag = true;
        // 固定日期
        if (planAccount.getPublishRate() == 1) {
            List<Integer> weeks = JSON.parseArray(planAccount.getPublishDate(), Integer.class);
            if (!weeks.contains(TimeZoneUtil.getTodayDayOfWeek(TimeZoneUtil.Timezone.china))) {
                planAccount.setStatus(2);
                planAccountMapper.updateByPrimaryKeySelective(planAccount);
                return false;
            }
        }
        // 指定时间
        if (isGzhGroupPushPlan && StringUtils.isNotEmpty(planAccount.getPublishPushTime())) {
            LocalTime currentTime = TimeZoneUtil.currentTime(TimeZoneUtil.Timezone.china);
            LocalTime pushTime = LocalTime.parse(planAccount.getPublishPushTime());
            if (currentTime.isBefore(pushTime.minusMinutes(20))) {
                return false;
            }
            if (currentTime.isAfter(pushTime.plusMinutes(60))) {
                return false;
            }
        } else if (StringUtils.isNotEmpty(planAccount.getPublishWindowStart())
                && StringUtils.isNotEmpty(planAccount.getPublishWindowEnd())) {
            if (!effectiveTime(planAccount.getPublishWindowStart(), planAccount.getPublishWindowEnd(),
                    TimeZoneUtil.Timezone.china)) {
                return false;
            }
            if (planAccount.getPublishTimeInterval() != null) {
                // 找到该账号最近发布的内容，判断时间间隔
                PublishContentExample example = new PublishContentExample();
                example.createCriteria().andStatusIn(Arrays.asList(1, 2))
                        .andCreateTimeGreaterThan(DateUtil.getThatDayDate())
                        .andPlanAccountIdEqualTo(planAccount.getId());
                example.setOrderByClause("create_time desc");
                List<PublishContent> publishContents = publishContentMapper.selectByExample(example);
                if (!CollectionUtils.isEmpty(publishContents)) {
                    long time = System.currentTimeMillis() - publishContents.get(0).getCreateTime().getTime();
                    if (time < planAccount.getPublishTimeInterval() * 60 * 1000L) {
                        return false;
                    }
                }
            }
        }
        int sendCount = publicContentService.getSendCount(planAccount.getId());
        if (isGzhGroupPushPlan) {
            //公众号群发每天只能成功发布一次
            if (sendCount >= 1) {
                return false;
            }
            //公众号群发每天最多尝试3次
            if (planAccount.getRetryCount() >= 3) {
                planAccount.setStatus(3);
                planAccountMapper.updateByPrimaryKeySelective(planAccount);
                LarkRobotUtil.sendMessage("计划发布失败3次，请查看 账号计划id为" + planAccount.getPlanId());
                return false;
            }
        }
        Integer publishNum = planAccount.getPublishNum();
        if (publishNum == null || publishNum == 0) {
            LarkRobotUtil.sendMessage("发布数量设置为0 planAccountId=" + planAccount.getId());
            return false;
        }
        if (sendCount >= publishNum) {
            return false;
        }

        int todayNeedSendCount = publishNum - sendCount;
        // 本次推送数量
        Integer prePushNum = planAccount.getPublishPreNum();
        if (Objects.isNull(prePushNum)) {
            prePushNum = 1;
        }
        //如果预计推送数量大于剩余推送数量  本次数量设置为剩余推送数量
        if (prePushNum > todayNeedSendCount) {
            prePushNum = todayNeedSendCount;
        }
        // 自动发布消息，每次暂时限定发布一条
        if (PublishGzhPushTypeEnum.robopost.getVal().equals(planAccount.getPushType())) {
            prePushNum = 1;
        }
        // 本次最少推送数量
        Integer preMinPushNum = planAccount.getPublishPreMinNum();
        if (Objects.isNull(preMinPushNum)) {
            preMinPushNum = prePushNum;
        }
        if (preMinPushNum > prePushNum) {
            preMinPushNum = prePushNum;
        }
        planAccount.setPublishPreNum(prePushNum);
        planAccount.setPublishPreMinNum(preMinPushNum);
        return flag;
    }


    @Override
    public void core() {
        List<PlanAccount> planAccounts = planAccountService.getNormalPlanAccount();
        if (CollectionUtils.isEmpty(planAccounts)) {
            return;
        }
        for (PlanAccount planAccount : planAccounts) {
            boolean flag = checkPlanAccount(planAccount);
            if (!flag) {
                continue;
            }
            ArticleSortRequest articleSortRequest = new ArticleSortRequest();
            articleSortRequest.setAccountId(planAccount.getAccountId());
            articleSortRequest.setAccountName(planAccount.getAccountName());
            articleSortRequest.setGhId(planAccount.getGhId());
            //测试参数
//            articleSortRequest.setStrategy("ArticleRankRandom");
//            articleSortRequest.setExcludeLog(true);
            articleSortRequest.setStrategy(planAccount.getSortStrategy());
            articleSortRequest.setPublishNum(planAccount.getPublishPreNum());
            if (Objects.equals(PublishGzhPushTypeEnum.auto_group_publish.getVal(), planAccount.getPushType())) {
                articleSortRequest.setPublishNum(planAccount.getPublishNum());
            }
            articleSortRequest.setPlanId(planAccount.getPlanId());
            articleSortRequest.setPushType(planAccount.getPushType());

            ArticleSortResponse articleSortResponse = sortService.publishArticleSort(articleSortRequest);
            if (articleSortResponse == null || articleSortResponse.getData() == null) {
                continue;
            }
            if (CollectionUtils.isEmpty(articleSortResponse.getData().getRank_list())) {
                continue;
            }
            ArticleSortResponseData data = articleSortResponse.getData();
            if (!CollectionUtils.isEmpty(data.getFilter_list())) {
                List<FilterSortContentParam> list = new ArrayList<>();
                for (PublishArticleData publishArticleData : data.getFilter_list()) {
                    FilterSortContentParam param = new FilterSortContentParam();
                    param.setFilterReason(publishArticleData.getFilterReason());
                    param.setPublishContentId(publishArticleData.getId());
                    list.add(param);
                }
                FilterSortParam param = new FilterSortParam();
                param.setFilterSortContentList(list);
                aigcService.filterSortContents(param);
            }
            List<LongArticleSystemPushContentParam> pushContentList = new ArrayList<>();
            //排序结束
            List<PublishArticleData> rankList = data.getRank_list();
            List<Long> sendIds = new ArrayList<>();
            for (PublishArticleData publishArticleData : rankList) {
                LongArticleSystemPushContentParam contentParam = new LongArticleSystemPushContentParam();
                PublishContent publishContent = publicContentService.getPublishContent(planAccount, publishArticleData);
                if (publishContent == null) {
                    continue;
                }
                sendIds.add(publishContent.getId());
                //获取小程序
                List<PublishMiniprogram> publishMiniprogramList = contentService.getPublishMiniprograms(publishArticleData);
                if (CollectionUtils.isEmpty(publishMiniprogramList)) {
                    List<CrawlerVideo> contentVideos =
                            contentService.getContentMiniVideo(publishArticleData, planAccount);
                    List<VideoDetail> videoDetails = new ArrayList<>();
                    //发布小程序
                    for (CrawlerVideo crawlerVideo : contentVideos) {
                        String videoOssPath = crawlerVideo.getVideoOssPath();
                        String platform = crawlerVideo.getPlatform();
                        String userId = crawlerVideo.getUserId();
                        String traceId = crawlerVideo.getTraceId();
                        String kimiTitle;
                        LongArticlesText kimiText = kimiService.getKimiText(crawlerVideo.getContentId());
                        if (kimiText != null && StringUtils.isNotEmpty(kimiText.getKimiTitle())) {
                            kimiTitle = kimiText.getKimiTitle();
                        } else {
                            kimiTitle = crawlerVideo.getVideoTitle();
                        }
                        VideoDetail publish = videoService.publish(videoOssPath, userId, kimiTitle, platform, traceId);
                        if (publish == null) {
                            continue;
                        }
                        videoDetails.add(publish);
                    }
                    if (videoDetails.size() < 2) {
                        MatchMiniprogramStatusParam statusParam = new MatchMiniprogramStatusParam();
                        statusParam.setStatus(3);
                        statusParam.setPublishContentId(publishContent.getPublishContentId());
                        statusParam.setErrorMsg("小程序视频数量不足");
                        aigcService.updateMatchMiniprogramStatus(statusParam);

                        PublishContent update = new PublishContent();
                        update.setId(publishContent.getId());
                        update.setStatus(3);
                        update.setReason("小程序数量不足");
                        publishContentMapper.updateByPrimaryKeySelective(update);
                        continue;
                    }
                    String ghId = planAccount.getGhId();
                    Long planAccountId = planAccount.getId();
                    publishMiniprogramList = cardService.generateCards(planAccountId, ghId, videoDetails,
                            planAccount.getMiniprogramUseType(), publishContent.getPublishContentId());
                    if (CollectionUtils.isEmpty(publishMiniprogramList) && publishMiniprogramList.size() < 2) {
                        continue;
                    }
                    for (PublishMiniprogram publishMiniprogram : publishMiniprogramList) {
                        String videoCover = publishMiniprogram.getVideoCover();
                        String wxUrl = aigcService.pushCover(videoCover, publishMiniprogram.getPublishContentId());
                        if (StringUtils.isEmpty(wxUrl)) {
                            continue;
                        }
                        publishMiniprogram.setWxUrl(wxUrl);
                        publishMiniprogram.setContentId(publishContent.getId());
                        RootSource rootSource = new RootSource();
                        rootSource.setAccountName(planAccount.getAccountName());
                        rootSource.setRootSourceId(publishMiniprogram.getRootSourceId());
                        rootSource.setGhId(planAccount.getGhId());
                        rootSource.setRequestTime(Integer.valueOf(String.valueOf(System.currentTimeMillis() / 1000)));
                        rootSource.setTraceId(publishMiniprogram.getTraceId());
                        rootSource.setPushType(2);
                        rootSource.setVideoId(publishMiniprogram.getVideoId());
                        rootSource.setContentId(publishArticleData.getSourceId());
                        rootSourceMapper.insertSelective(rootSource);
                        publishMiniprogramMapper.insertSelective(publishMiniprogram);
                    }
                }

                List<PublishMiniprogramParam> publishCardList = new ArrayList<>();
                for (PublishMiniprogram publishMiniprogram : publishMiniprogramList) {
                    PublishMiniprogramParam param = new PublishMiniprogramParam();
                    param.setProgramId(publishMiniprogram.getAppId());
                    param.setProgramName(publishMiniprogram.getAppName());
                    param.setProgramAvatar(publishMiniprogram.getAvatar());
                    param.setMini_id(publishMiniprogram.getMiniId());
                    param.setMiniprogramUseType(publishMiniprogram.getMiniProgramType());
                    String videoTitle = publishMiniprogram.getVideoTitle();
                    if (org.springframework.util.StringUtils.hasText(videoTitle)) {
                        videoTitle = videoTitle.replace("\"", "");
                        if (videoTitle.length() > 32) {
                            videoTitle = videoTitle.substring(0, 32);
                        }
                        param.setProductionName(videoTitle);
                    }
                    String wxUrl = null;
                    if (StringUtils.isNotEmpty(publishMiniprogram.getWxUrl())) {
                        wxUrl = publishMiniprogram.getWxUrl();
                    } else {
                        aigcService.pushCover(publishMiniprogram.getVideoCover(), publishMiniprogram.getPublishContentId());
                    }
                    if (StringUtils.isEmpty(wxUrl)) {
                        continue;
                    }
                    param.setProductionCover(wxUrl);
                    param.setProductionPath(publishMiniprogram.getProductionPath());
                    param.setRootShareId(publishMiniprogram.getRootShareId());
                    param.setSource(publishMiniprogram.getSource());
                    param.setVideoUrl(publishMiniprogram.getVideoPath());
                    param.setParagraphPosition(BigDecimal.valueOf(publishMiniprogram.getIndex() * 0.25));
                    if (publishMiniprogram.getIndex() == 1) {
                        param.setParagraphPosition(BigDecimal.valueOf(0.01));
                    }
                    param.setTraceId(publishMiniprogram.getTraceId());
                    publishCardList.add(param);
                }
                if (publishCardList.size() < 2) {
                    MatchMiniprogramStatusParam statusParam = new MatchMiniprogramStatusParam();
                    statusParam.setStatus(3);
                    statusParam.setPublishContentId(publishContent.getPublishContentId());
                    statusParam.setErrorMsg("小程序视频数量不足");
                    aigcService.updateMatchMiniprogramStatus(statusParam);

                    PublishContent update = new PublishContent();
                    update.setId(publishContent.getId());
                    update.setStatus(3);
                    update.setReason("小程序数量不足");
                    publishContentMapper.updateByPrimaryKeySelective(update);
                    continue;
                }
                contentParam.setPublishContentId(publishContent.getPublishContentId());
                contentParam.setMiniprogramCardList(publishCardList);
                pushContentList.add(contentParam);
            }
            LongArticleSystemCreatePushTaskParam gzhPushParam = new LongArticleSystemCreatePushTaskParam();
            gzhPushParam.setPlanId(planAccount.getPlanId());
            gzhPushParam.setAccountId(planAccount.getAccountId());
            gzhPushParam.setPushType(planAccount.getPushType());
            gzhPushParam.setPushScheduleTimestamp(System.currentTimeMillis());
            if (StringUtils.isNotEmpty(planAccount.getPublishPushTime())) {
                String dateTimeStr = TimeZoneUtil.todayYMD(TimeZoneUtil.Timezone.china) + " " + planAccount.getPublishPushTime();
                gzhPushParam.setPushScheduleTimestamp(TimeZoneUtil.getDateStrTimestamp(dateTimeStr, "yyyy-MM-dd HH:mm:ss", TimeZoneUtil.Timezone.china));
            } else {
                gzhPushParam.setPushScheduleTimestamp(System.currentTimeMillis());
            }
            gzhPushParam.setPushContentList(pushContentList);
            if (Objects.equals(planAccount.getPushType(), PublishGzhPushTypeEnum.auto_group_publish.getVal())
                    && pushContentList.size() < planAccount.getPublishPreMinNum()) {
                continue;
            }
            log.info("gzhPushParam={}", gzhPushParam);
            String pushId = aigcService.createPushTask(gzhPushParam);
            log.info("pushId = {}", pushId);
            if (StringUtils.isNotEmpty(pushId)) {
                for (Long id : sendIds) {
                    PublishContent publishContent = publishContentMapper.selectByPrimaryKey(id);
                    if (publishContent == null || publishContent.getStatus() != 0) {
                        continue;
                    }
                    PublishContent update = new PublishContent();
                    update.setId(id);
                    update.setPushId(pushId);
                    update.setStatus(1);
                    publishContentMapper.updateByPrimaryKeySelective(update);
                }
            }
        }
    }

    @Override
    public void getPushStatus() {
        PublishContentExample example = new PublishContentExample();
        example.createCriteria().andStatusEqualTo(1).andPushIdIsNotNull();
        List<PublishContent> publishContents = publishContentMapper.selectByExample(example);
        Set<String> collect = publishContents.stream().map(PublishContent::getPushId).collect(Collectors.toSet());
        for (String pushId : collect) {
            PushStatusParam param = new PushStatusParam();
            param.setPushId(pushId);
            PushStatusVO pushStatusVO = aigcService.queryPushStatus(param);
            log.info("pushStatusVO={} param={}", pushStatusVO, param);
            if (pushStatusVO == null) {
                continue;
            }
            if (StringUtils.isNotEmpty(pushStatusVO.getErrorMsg())) {
                String accountName = "";
                PublishContentExample publishContentExample = new PublishContentExample();
                publishContentExample.createCriteria().andPushIdEqualTo(pushId);
                List<PublishContent> failPublishContents = publishContentMapper.selectByExample(publishContentExample);
                if (!CollectionUtils.isEmpty(failPublishContents)) {
                    Long planAccountId = failPublishContents.get(0).getPlanAccountId();
                    PlanAccount planAccount = planAccountMapper.selectByPrimaryKey(planAccountId);
                    if (planAccount != null && StringUtils.isNotEmpty(planAccount.getAccountName())) {
                        accountName = planAccount.getAccountName();
                    }
                }
                log.error("push error pushId={} msg={}", pushId, pushStatusVO.getErrorMsg());
                LarkRobotUtil.sendMessage("发布失败，请查看，pushId=" + pushId + "失败信息"
                        + pushStatusVO.getErrorMsg() + "账号名称：" + accountName);
            }
            int updateStatus = 0;
            //推送失败  重新推送
            if (pushStatusVO.getPushStatus() == 3) {
                updateStatus = 3;
            }
            //推送成功
            if (pushStatusVO.getPushStatus() == 2) {
                //发表失败
                if (pushStatusVO.getPublishStatus() == 3) {
                    updateStatus = 3;
                    if (StringUtils.isNotEmpty(pushStatusVO.getErrorCode())) {
                        int errorCode = Integer.parseInt(pushStatusVO.getErrorCode());
                        if (errorCode == 6 || errorCode == 7 || errorCode == 8 || errorCode == 999) {
                            updateStatus = 2;
                        }
                    }

                }
                //发表成功
                if (pushStatusVO.getPublishStatus() == 2) {
                    updateStatus = 2;
                }
            }
            //更新状态  2成功  3失败
            if (updateStatus != 0) {
                PublishContentExample publishContentExample = new PublishContentExample();
                publishContentExample.createCriteria().andPushIdEqualTo(pushId);
                PublishContent update = new PublishContent();
                if (StringUtils.isNotEmpty(pushStatusVO.getErrorMsg())) {
                    update.setReason(pushStatusVO.getErrorMsg());
                }
                update.setStatus(updateStatus);
                publishContentMapper.updateByExampleSelective(update, publishContentExample);

                List<PublishContent> publishContentList = publishContentMapper.selectByExample(publishContentExample);
                if (!CollectionUtils.isEmpty(publishContentList)) {
                    Long planAccountId = publishContentList.get(0).getPlanAccountId();
                    if (planAccountId == null) {
                        continue;
                    }
                    PlanAccount planAccount = planAccountMapper.selectByPrimaryKey(planAccountId);
                    if (Objects.equals(planAccount.getPushType(), PublishGzhPushTypeEnum.auto_group_publish.getVal())) {
                        if (updateStatus == 2) {
                            planAccount.setStatus(2);
                            planAccountMapper.updateByPrimaryKeySelective(planAccount);
                        }
                        if (updateStatus == 3) {
                            planAccount.setRetryCount(planAccount.getRetryCount() + 1);
                            planAccountMapper.updateByPrimaryKeySelective(planAccount);
                        }
                    }
                }
            }
        }
    }
}
