package com.tzld.piaoquan.longarticle.service.remote.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.tzld.piaoquan.longarticle.model.vo.LongArticleSystemPlan;
import com.tzld.piaoquan.longarticle.model.vo.*;
import com.tzld.piaoquan.longarticle.service.remote.AigcService;
import com.tzld.piaoquan.longarticle.utils.HttpClientUtil;
import com.tzld.piaoquan.longarticle.utils.HttpPoolClientUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.List;

@Slf4j
@Service
public class AigcServiceImpl implements AigcService {

    private static final HttpPoolClientUtil HTTP_POOL_CLIENT_UTIL_DEFAULT = HttpClientUtil.create(5000, 5000, 20, 100, 3, 3000);

    @Override
    //获取所有长文系统托管计划
    public List<LongArticleSystemPlan> getAllLongArticleSystemPlan() {
        String apiUrl = "http://aigc-api.cybertogether.net/aigc/publish/LongArticleSystem/getAllLongArticleSystemPlan";
        try {
            String res = HTTP_POOL_CLIENT_UTIL_DEFAULT.post(apiUrl);
            JSONObject jsonObject = JSON.parseObject(res);
            Integer code = jsonObject.getInteger("code");
            if (code == 0) {
                JSONArray data = jsonObject.getJSONArray("data");
                return data.toJavaList(LongArticleSystemPlan.class);
            }
            return null;
        } catch (Exception e) {
            log.error("getAllLongArticleSystemPlan error", e);
        }
        return null;
    }

    @Override
    public LongArticleSystemContentVO getContentItemList(LongArticleSystemGetContentsParam param) {
        int pageNum = 1;
        int pageSize = 500;
        param.setPageNum(pageNum);
        param.setPageSize(pageSize);
        LongArticleSystemContentVO longArticleSystemContentVO = listWaitingMatchMiniprogramContents(param);
        if (longArticleSystemContentVO == null) {
            return null;
        }
        Long totalCount = longArticleSystemContentVO.getTotalCount();
        while ((long) pageNum * pageSize < totalCount) {
            pageNum++;
            param.setPageNum(pageNum);
            LongArticleSystemContentVO longArticleSystemContentVO1 = listWaitingMatchMiniprogramContents(param);
            if (longArticleSystemContentVO1 != null && !CollectionUtils.isEmpty(longArticleSystemContentVO.getContentItemList())) {
                longArticleSystemContentVO.getContentItemList().addAll(longArticleSystemContentVO1.getContentItemList());
            }
        }
        return longArticleSystemContentVO;
    }

    @Override
    //分页获取待排序内容列表
    public LongArticleSystemContentVO listWaitingMatchMiniprogramContents(LongArticleSystemGetContentsParam param) {
        String apiUrl = "http://aigc-api.cybertogether.net/aigc/publish/LongArticleSystem/listWaitingMatchMiniprogramContents";
        try {
            String res = HTTP_POOL_CLIENT_UTIL_DEFAULT.post(apiUrl, JSON.toJSONString(param));
            JSONObject jsonObject = JSON.parseObject(res);
            Integer code = jsonObject.getInteger("code");
            if (code == 0) {
                return jsonObject.getJSONObject("data").toJavaObject(LongArticleSystemContentVO.class);
            }
        } catch (Exception e) {
            log.error("listWaitingMatchMiniprogramContents error", e);
        }
        return null;
    }

    @Override
    //创建推送任务
    public String createPushTask(LongArticleSystemCreatePushTaskParam param) {
        String apiUrl = "http://aigc-api.cybertogether.net/aigc/publish/LongArticleSystem/createPushTask";
        try {
            String res = HTTP_POOL_CLIENT_UTIL_DEFAULT.post(apiUrl, JSON.toJSONString(param));
            JSONObject jsonObject = JSON.parseObject(res);
            Integer code = jsonObject.getInteger("code");
            if (code == 0) {
                return jsonObject.getJSONObject("data").getString("pushId");
            }
        } catch (Exception e) {
            log.error("listWaitingMatchMiniprogramContents error", e);
        }
        return "";
    }

    @Override
    //更新匹配小程序状态
    public boolean updateMatchMiniprogramStatus(MatchMiniprogramStatusParam param) {
        String apiUrl = "http://aigc-api.cybertogether.net/aigc/publish/LongArticleSystem/updateMatchMiniprogramStatus";
        try {
            String res = HTTP_POOL_CLIENT_UTIL_DEFAULT.post(apiUrl, JSON.toJSONString(param));
            JSONObject jsonObject = JSON.parseObject(res);
            Integer code = jsonObject.getInteger("code");
            if (code == 0) {
                return true;
            }
        } catch (Exception e) {
            log.error("updateMatchMiniprogramStatus error", e);
        }
        return false;
    }

    @Override
    public boolean filterSortContents(FilterSortParam param) {
        String apiUrl = "http://aigc-api.cybertogether.net/aigc/publish/LongArticleSystem/filterSortContents";
        try {
            String res = HTTP_POOL_CLIENT_UTIL_DEFAULT.post(apiUrl, JSON.toJSONString(param));
            JSONObject jsonObject = JSON.parseObject(res);
            Integer code = jsonObject.getInteger("code");
            if (code == 0) {
                return true;
            }
        } catch (Exception e) {
            log.error("filterSortContents error", e);
        }
        return false;
    }

    @Override
    public PushStatusVO queryPushStatus(PushStatusParam param) {
        String apiUrl = "http://aigc-api.cybertogether.net/aigc/publish/LongArticleSystem/queryPushStatus";
        try {
            String res = HTTP_POOL_CLIENT_UTIL_DEFAULT.post(apiUrl, JSON.toJSONString(param));
            JSONObject jsonObject = JSON.parseObject(res);
            Integer code = jsonObject.getInteger("code");
            if (code == 0) {
                return jsonObject.getJSONObject("data").toJavaObject(PushStatusVO.class);
            }
        } catch (Exception e) {
            log.error("queryPushStatus error", e);
        }
        return null;
    }

    @Override
    public String pushCover(String coverUrl, String publishContentId) {
        String apiUrl = "http://aigc-api.cybertogether.net/aigc//publish/LongArticleSystem/uploadMiniprogramCover";
        try {
            JSONObject param = new JSONObject();
            param.put("coverUrl", coverUrl);
            param.put("publishContentId", publishContentId);
            String res = HTTP_POOL_CLIENT_UTIL_DEFAULT.post(apiUrl, JSONObject.toJSONString(param));
            JSONObject jsonObject = JSON.parseObject(res);
            Integer code = jsonObject.getInteger("code");
            if (code == 0) {
                JSONObject data = jsonObject.getJSONObject("data");
                return data.getString("wxUrl");
            }
            return null;
        } catch (Exception e) {
            log.error("getAllLongArticleSystemPlan error", e);
        }
        return null;
    }
}
