package com.tzld.piaoquan.longarticle.utils;

import org.apache.commons.codec.digest.DigestUtils;
import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.message.BasicHeader;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Objects;
import java.util.UUID;

public class VideoDownloader {

    public static String downloadCover(String filePath, String platform, String coverUrl) {
        try {
            HttpResponse response = sendRequest(platform, coverUrl, "cover");
            if (response.getStatusLine().getStatusCode() != 200) {
                return null;
            }

            try (InputStream inputStream = response.getEntity().getContent();
                 FileOutputStream outputStream = new FileOutputStream(new File(filePath))) {
                byte[] buffer = new byte[1024];
                int bytesRead;
                while ((bytesRead = inputStream.read(buffer)) != -1) {
                    outputStream.write(buffer, 0, bytesRead);
                }
            }
            return filePath;
        } catch (Exception e) {

        }
        return "";
    }

    public static HttpResponse sendRequest(String platform, String url, String downloadType) throws IOException {
        CloseableHttpClient httpClient = HttpClients.createDefault();
        HttpGet request = new HttpGet(url);
        request.setHeaders(requestHeader(platform, url, downloadType));
        return httpClient.execute(request);
    }

    public static org.apache.http.Header[] requestHeader(String platform, String url, String downloadType) {
        switch (platform) {
            case "xg_search":
                // 根据 URL 设置不同的请求头
                if (url.contains("v9-xg-web-pc.ixigua.com")) {
                    return new org.apache.http.Header[]{
                            new BasicHeader("Accept", "*/*"),
                            new BasicHeader("Accept-Language", "zh-CN,zh;q=0.9"),
                            new BasicHeader("Host", "v9-xg-web-pc.ixigua.com"),
                            new BasicHeader("User-Agent", "Mozilla/5.0"),
                            new BasicHeader("Origin", "https://www.ixigua.com/"),
                            new BasicHeader("Referer", "https://www.ixigua.com/")
                    };
                } else if (url.contains("v3-xg-web-pc.ixigua.com")) {
                    return new org.apache.http.Header[]{
                            new BasicHeader("Accept", "*/*"),
                            new BasicHeader("Accept-Language", "zh-CN,zh;q=0.9"),
                            new BasicHeader("Host", "v3-xg-web-pc.ixigua.com"),
                            new BasicHeader("User-Agent", "Mozilla/5.0"),
                            new BasicHeader("Origin", "https://www.ixigua.com/"),
                            new BasicHeader("Referer", "https://www.ixigua.com/")
                    };
                } else if (Objects.equals(downloadType, "cover")) {
                    return new org.apache.http.Header[]{
                            new BasicHeader("Accept", "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7"),
                            new BasicHeader("Accept-Language", "en,zh;q=0.9,zh-CN;q=0.8"),
                            new BasicHeader("Cache-Control", "max-age=0"),
                            new BasicHeader("Proxy-Connection", "keep-alive"),
                            new BasicHeader("Upgrade-Insecure-Requests", "1"),
                            new BasicHeader("User-Agent", "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/125.0.0.0 Safari/537.36")
                    };
                }
                // 其他 headers 根据 downloadType 处理
                break;
            case "baidu_search":
                return new org.apache.http.Header[]{
                        new BasicHeader("Accept", "*/*"),
                        new BasicHeader("Accept-Language", "zh-CN,zh;q=0.9"),
                        new BasicHeader("User-Agent", "Mozilla/5.0")
                };
            case "wx_search":
                return new org.apache.http.Header[]{
                        new BasicHeader("Accept", "*/*"),
                        new BasicHeader("Accept-Language", "zh-CN,zh;q=0.9"),
                        new BasicHeader("User-Agent", "Mozilla/5.0"),
                        new BasicHeader("Origin", "https://mp.weixin.qq.com"),
                        new BasicHeader("Referer", "https://mp.weixin.qq.com")
                };
            case "dy_search":
                return new org.apache.http.Header[]{
                        new BasicHeader("accept", "*/*"),
                        new BasicHeader("accept-language", "en,zh;q=0.9,zh-CN;q=0.8"),
                        new BasicHeader("priority", "i"),
                        new BasicHeader("referer", "https://v11-coldf.douyinvod.com/"),
                        new BasicHeader("user-agent", "Mozilla/5.0")
                };
            default:
                return new org.apache.http.Header[]{};
        }
        return new org.apache.http.Header[]{};
    }

    public static String downloadVideo(String filePath, String platform, String videoUrl, String downloadType) {
        try {
            HttpResponse response = sendRequest(platform, videoUrl, downloadType);
            if (response.getStatusLine().getStatusCode() != 200 && response.getStatusLine().getStatusCode() != 206) {
                System.out.println("Error: " + response.getStatusLine().getStatusCode());
                return null;
            }

            try (InputStream inputStream = response.getEntity().getContent();
                 FileOutputStream outputStream = new FileOutputStream(new File(filePath))) {
                byte[] buffer = new byte[1024 * 1024];
                int bytesRead;
                while ((bytesRead = inputStream.read(buffer)) != -1) {
                    outputStream.write(buffer, 0, bytesRead);
                }
            }
            return filePath;
        } catch (Exception e) {
            System.out.println(e.getMessage());
        }
        return "";
    }

    public static String generateVideoPath(String platform, String videoId) {
        String index = String.format("%s-%s-%s", platform, videoId, UUID.randomUUID());
        String md5Hash = DigestUtils.md5Hex(index);
        String fileName = String.format("%s.mp4", md5Hash);
        String filePath = String.join(File.separator, System.getProperty("user.dir"), fileName);
        return filePath; // 返回文件路径和封面路径
    }

    public static String generateCoverPath(String platform, String videoId) {
        String index = String.format("%s-%s-%s", platform, videoId, UUID.randomUUID());
        String md5Hash = DigestUtils.md5Hex(index);
        String coverName = String.format("%s.png", md5Hash);
        String coverPath = String.join(File.separator, System.getProperty("user.dir"), coverName);
        return coverPath; // 返回文件路径和封面路径
    }

    public static void main(String[] args) {
        // 示例用法
        String coverUrl = "https://example.com/cover.jpg";
        String filePath = "path/to/cover.png";
        downloadCover(filePath, "xg_search", coverUrl);
        String videoUrl = "https://example.com/video.mp4";
        String videoFilePath = "path/to/video.mp4";
        downloadVideo(videoFilePath, "xg_search", videoUrl, "video");
        String paths = generateVideoPath("xg_search", "videoId");
        System.out.println("Generated paths: " + paths);

    }
}